# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel

__all__ = ["ApplicationGraphsResponse"]


class ApplicationGraphsResponse(BaseModel):
    graph_ids: List[str]
    """A list of Knowledge Graphs associated with the application."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["TranslationTranslateParams"]


class TranslationTranslateParams(TypedDict, total=False):
    formality: Required[bool]
    """Whether to use formal or informal language in the translation.

    See the
    [list of languages that support formality](https://dev.writer.com/api-reference/translation-api/language-support#formality).
    If the language does not support formality, this parameter is ignored.
    """

    length_control: Required[bool]
    """Whether to control the length of the translated text.

    See the
    [list of languages that support length control](https://dev.writer.com/api-reference/translation-api/language-support#length-control).
    If the language does not support length control, this parameter is ignored.
    """

    mask_profanity: Required[bool]
    """Whether to mask profane words in the translated text.

    See the
    [list of languages that do not support profanity masking](https://dev.writer.com/api-reference/translation-api/language-support#profanity-masking).
    If the language does not support profanity masking, this parameter is ignored.
    """

    model: Required[Literal["palmyra-translate"]]
    """The model to use for translation."""

    source_language_code: Required[str]
    """
    The [ISO-639-1](https://en.wikipedia.org/wiki/List_of_ISO_639_language_codes)
    language code of the original text to translate. For example, `en` for English,
    `zh` for Chinese, `fr` for French, `es` for Spanish. If the language has a
    variant, the code appends the two-digit
    [ISO-3166 country code](https://en.wikipedia.org/wiki/List_of_ISO_3166_country_codes).
    For example, Mexican Spanish is `es-MX`. See the
    [list of supported languages and language codes](https://dev.writer.com/api-reference/translation-api/language-support).
    """

    target_language_code: Required[str]
    """
    The [ISO-639-1](https://en.wikipedia.org/wiki/List_of_ISO_639_language_codes)
    language code of the target language for the translation. For example, `en` for
    English, `zh` for Chinese, `fr` for French, `es` for Spanish. If the language
    has a variant, the code appends the two-digit
    [ISO-3166 country code](https://en.wikipedia.org/wiki/List_of_ISO_3166_country_codes).
    For example, Mexican Spanish is `es-MX`. See the
    [list of supported languages and language codes](https://dev.writer.com/api-reference/translation-api/language-support).
    """

    text: Required[str]
    """The text to translate. Maximum of 100,000 words."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["VisionAnalyzeParams", "Variable"]


class VisionAnalyzeParams(TypedDict, total=False):
    model: Required[Literal["palmyra-vision"]]
    """The model to use for image analysis."""

    prompt: Required[str]
    """The prompt to use for the image analysis.

    The prompt must include the name of each image variable, surrounded by double
    curly braces (`{{}}`). For example,
    `Describe the difference between the image {{image_1}} and the image {{image_2}}`.
    """

    variables: Required[Iterable[Variable]]


class Variable(TypedDict, total=False):
    file_id: Required[str]
    """The File ID of the image to analyze.

    The file must be uploaded to the Writer platform before it can be used in a
    vision request.
    """

    name: Required[str]
    """The name of the file variable.

    You must reference this name in the prompt with double curly braces (`{{}}`).
    For example,
    `Describe the difference between the image {{image_1}} and the image {{image_2}}`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["JobListParams"]


class JobListParams(TypedDict, total=False):
    limit: int
    """The pagination limit for retrieving the jobs."""

    offset: int
    """The pagination offset for retrieving the jobs."""

    status: Literal["in_progress", "failed", "completed"]
    """The status of the job."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .function_params import FunctionParams

__all__ = ["FunctionDefinition"]


class FunctionDefinition(BaseModel):
    name: str
    """Name of the function."""

    description: Optional[str] = None
    """Description of the function."""

    parameters: Optional[FunctionParams] = None
    """The parameters of the function."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["ApplicationGenerateContentChunk", "Delta", "DeltaStage"]


class DeltaStage(BaseModel):
    id: str
    """The unique identifier for the stage."""

    content: str
    """The text content of the stage."""

    sources: Optional[List[str]] = None
    """A list of sources (URLs) that that stage used to process that particular step."""


class Delta(BaseModel):
    content: Optional[str] = None
    """The main text output."""

    stages: Optional[List[DeltaStage]] = None
    """A list of stages that show the 'thinking process'."""

    title: Optional[str] = None
    """The name of the output."""


class ApplicationGenerateContentChunk(BaseModel):
    delta: Delta



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ToolCall", "Function"]


class Function(TypedDict, total=False):
    arguments: Required[str]

    name: str


class ToolCall(TypedDict, total=False):
    id: Required[str]

    function: Required[Function]

    type: Required[Literal["function"]]

    index: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["CompletionChunk"]


class CompletionChunk(BaseModel):
    value: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["GraphAddFileToGraphParams"]


class GraphAddFileToGraphParams(TypedDict, total=False):
    file_id: Required[str]
    """The unique identifier of the file."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = [
    "ComprehendMedicalResponse",
    "Entity",
    "EntityAttribute",
    "EntityAttributeConcept",
    "EntityAttributeTrait",
    "EntityConcept",
    "EntityTrait",
]


class EntityAttributeConcept(BaseModel):
    code: str

    description: str

    score: float


class EntityAttributeTrait(BaseModel):
    name: str

    score: float


class EntityAttribute(BaseModel):
    begin_offset: int

    concepts: List[EntityAttributeConcept]

    end_offset: int

    relationship_score: float

    score: float

    text: str

    traits: List[EntityAttributeTrait]

    type: str

    category: Optional[str] = None

    relationship_type: Optional[str] = None


class EntityConcept(BaseModel):
    code: str

    description: str

    score: float


class EntityTrait(BaseModel):
    name: str

    score: float


class Entity(BaseModel):
    attributes: List[EntityAttribute]

    begin_offset: int

    category: str

    concepts: List[EntityConcept]

    end_offset: int

    score: float

    text: str

    traits: List[EntityTrait]

    type: str


class ComprehendMedicalResponse(BaseModel):
    entities: List[Entity]
    """An array of medical entities extracted from the input text."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["ToolWebSearchResponse", "Source"]


class Source(BaseModel):
    raw_content: Optional[str] = None
    """Raw content from the source URL.

    Not included if `include_raw_content` is `false`.
    """

    url: Optional[str] = None
    """URL of the search result."""


class ToolWebSearchResponse(BaseModel):
    query: str
    """The search query that was submitted."""

    sources: List[Source]
    """The search results found."""

    answer: Optional[str] = None
    """Generated answer based on the search results.

    Not included if `include_answer` is `false`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict

from ..._models import BaseModel

__all__ = ["ToolChoiceJsonObject"]


class ToolChoiceJsonObject(BaseModel):
    value: Dict[str, object]
    """A JSON object that specifies the tool to call.

    For example, `{"type": "function", "function": {"name": "get_current_weather"}}`
    """



from typing import Union
from typing_extensions import TypeAlias

from .response_format_text import ResponseFormatText
from .response_format_json_object import ResponseFormatJSONObject
from .response_format_json_schema import ResponseFormatJSONSchema

__all__ = ["ResponseFormat"]

ResponseFormat: TypeAlias = Union[ResponseFormatText, ResponseFormatJSONObject, ResponseFormatJSONSchema]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .file import File as File
from .graph import Graph as Graph
from .shared import (
    Source as Source,
    Logprobs as Logprobs,
    ToolCall as ToolCall,
    GraphData as GraphData,
    ToolParam as ToolParam,
    ErrorObject as ErrorObject,
    ErrorMessage as ErrorMessage,
    LogprobsToken as LogprobsToken,
    FunctionParams as FunctionParams,
    ToolChoiceString as ToolChoiceString,
    ToolCallStreaming as ToolCallStreaming,
    FunctionDefinition as FunctionDefinition,
    ToolChoiceJsonObject as ToolChoiceJsonObject,
)
from .question import Question as Question
from .completion import Completion as Completion
from .chat_completion import ChatCompletion as ChatCompletion
from .vision_response import VisionResponse as VisionResponse
from .chat_chat_params import ChatChatParams as ChatChatParams
from .completion_chunk import CompletionChunk as CompletionChunk
from .file_list_params import FileListParams as FileListParams
from .file_retry_params import FileRetryParams as FileRetryParams
from .graph_list_params import GraphListParams as GraphListParams
from .file_upload_params import FileUploadParams as FileUploadParams
from .file_retry_response import FileRetryResponse as FileRetryResponse
from .graph_create_params import GraphCreateParams as GraphCreateParams
from .graph_update_params import GraphUpdateParams as GraphUpdateParams
from .model_list_response import ModelListResponse as ModelListResponse
from .file_delete_response import FileDeleteResponse as FileDeleteResponse
from .translation_response import TranslationResponse as TranslationResponse
from .chat_completion_chunk import ChatCompletionChunk as ChatCompletionChunk
from .chat_completion_usage import ChatCompletionUsage as ChatCompletionUsage
from .graph_create_response import GraphCreateResponse as GraphCreateResponse
from .graph_delete_response import GraphDeleteResponse as GraphDeleteResponse
from .graph_question_params import GraphQuestionParams as GraphQuestionParams
from .graph_update_response import GraphUpdateResponse as GraphUpdateResponse
from .tool_ai_detect_params import ToolAIDetectParams as ToolAIDetectParams
from .tool_parse_pdf_params import ToolParsePdfParams as ToolParsePdfParams
from .vision_analyze_params import VisionAnalyzeParams as VisionAnalyzeParams
from .chat_completion_choice import ChatCompletionChoice as ChatCompletionChoice
from .tool_web_search_params import ToolWebSearchParams as ToolWebSearchParams
from .application_list_params import ApplicationListParams as ApplicationListParams
from .chat_completion_message import ChatCompletionMessage as ChatCompletionMessage
from .question_response_chunk import QuestionResponseChunk as QuestionResponseChunk
from .tool_ai_detect_response import ToolAIDetectResponse as ToolAIDetectResponse
from .tool_parse_pdf_response import ToolParsePdfResponse as ToolParsePdfResponse
from .completion_create_params import CompletionCreateParams as CompletionCreateParams
from .tool_web_search_response import ToolWebSearchResponse as ToolWebSearchResponse
from .application_list_response import ApplicationListResponse as ApplicationListResponse
from .translation_translate_params import TranslationTranslateParams as TranslationTranslateParams
from .application_retrieve_response import ApplicationRetrieveResponse as ApplicationRetrieveResponse
from .graph_add_file_to_graph_params import GraphAddFileToGraphParams as GraphAddFileToGraphParams
from .application_generate_content_chunk import ApplicationGenerateContentChunk as ApplicationGenerateContentChunk
from .application_generate_content_params import ApplicationGenerateContentParams as ApplicationGenerateContentParams
from .tool_context_aware_splitting_params import ToolContextAwareSplittingParams as ToolContextAwareSplittingParams
from .application_generate_content_response import (
    ApplicationGenerateContentResponse as ApplicationGenerateContentResponse,
)
from .graph_remove_file_from_graph_response import GraphRemoveFileFromGraphResponse as GraphRemoveFileFromGraphResponse
from .tool_context_aware_splitting_response import (
    ToolContextAwareSplittingResponse as ToolContextAwareSplittingResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .shared.logprobs import Logprobs
from .shared.graph_data import GraphData
from .chat_completion_usage import ChatCompletionUsage
from .chat_completion_message import ChatCompletionMessage
from .shared.tool_call_streaming import ToolCallStreaming

__all__ = ["ChatCompletionChunk", "Choice", "ChoiceDelta", "ChoiceDeltaLlmData", "ChoiceDeltaTranslationData"]


class ChoiceDeltaLlmData(BaseModel):
    model: str
    """The model used by the tool."""

    prompt: str
    """The prompt processed by the model."""


class ChoiceDeltaTranslationData(BaseModel):
    source_language_code: str
    """The language code of the source text."""

    source_text: str
    """The text the tool translated."""

    target_language_code: str
    """The language code of the target text."""


class ChoiceDelta(BaseModel):
    content: Optional[str] = None
    """The text content produced by the model.

    This field contains the actual output generated, reflecting the model's response
    to the input query or command.
    """

    graph_data: Optional[GraphData] = None

    llm_data: Optional[ChoiceDeltaLlmData] = None

    refusal: Optional[str] = None

    role: Optional[Literal["user", "assistant", "system"]] = None
    """
    Specifies the role associated with the content, indicating whether the message
    is from the 'assistant' or another defined role, helping to contextualize the
    output within the interaction flow.
    """

    tool_calls: Optional[List[ToolCallStreaming]] = None

    translation_data: Optional[ChoiceDeltaTranslationData] = None


class Choice(BaseModel):
    delta: ChoiceDelta
    """A chat completion delta generated by streamed model responses."""

    finish_reason: Optional[Literal["stop", "length", "content_filter", "tool_calls"]] = None
    """Describes the condition under which the model ceased generating content.

    Common reasons include 'length' (reached the maximum output size), 'stop'
    (encountered a stop sequence), 'content_filter' (harmful content filtered out),
    or 'tool_calls' (encountered tool calls).
    """

    index: int
    """The index of the choice in the list of completions generated by the model."""

    logprobs: Optional[Logprobs] = None
    """Log probability information for the choice."""

    message: Optional[ChatCompletionMessage] = None
    """The chat completion message from the model.

    Note: this field is deprecated for streaming. Use `delta` instead.
    """


class ChatCompletionChunk(BaseModel):
    id: str
    """A globally unique identifier (UUID) for the response generated by the API.

    This ID can be used to reference the specific operation or transaction within
    the system for tracking or debugging purposes.
    """

    choices: List[Choice]
    """
    An array of objects representing the different outcomes or results produced by
    the model based on the input provided.
    """

    created: int
    """The Unix timestamp (in seconds) when the response was created.

    This timestamp can be used to verify the timing of the response relative to
    other events or operations.
    """

    model: str
    """Identifies the specific model used to generate the response."""

    object: Literal["chat.completion.chunk"]
    """
    The type of object returned, which is always `chat.completion.chunk` for
    streaming chat responses.
    """

    service_tier: Optional[str] = None

    system_fingerprint: Optional[str] = None

    usage: Optional[ChatCompletionUsage] = None
    """Usage information for the chat completion response.

    Please note that at this time Knowledge Graph tool usage is not included in this
    object.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["ChatCompletionUsage", "CompletionTokensDetails", "PromptTokenDetails"]


class CompletionTokensDetails(BaseModel):
    reasoning_tokens: int


class PromptTokenDetails(BaseModel):
    cached_tokens: int


class ChatCompletionUsage(BaseModel):
    completion_tokens: int

    prompt_tokens: int

    total_tokens: int

    completion_tokens_details: Optional[CompletionTokensDetails] = None

    prompt_token_details: Optional[PromptTokenDetails] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .shared.tool_call import ToolCall
from .shared.graph_data import GraphData

__all__ = ["ChatCompletionMessage", "LlmData", "TranslationData", "WebSearchData", "WebSearchDataSource"]


class LlmData(BaseModel):
    model: str
    """The model used by the tool."""

    prompt: str
    """The prompt processed by the model."""


class TranslationData(BaseModel):
    source_language_code: str
    """The language code of the source text."""

    source_text: str
    """The text the tool translated."""

    target_language_code: str
    """The language code of the target text."""


class WebSearchDataSource(BaseModel):
    raw_content: Optional[str] = None

    url: Optional[str] = None


class WebSearchData(BaseModel):
    sources: List[WebSearchDataSource]


class ChatCompletionMessage(BaseModel):
    content: str
    """The text content produced by the model.

    This field contains the actual output generated, reflecting the model's response
    to the input query or command.
    """

    refusal: Optional[str] = None

    role: Literal["assistant"]
    """Specifies the role associated with the content."""

    graph_data: Optional[GraphData] = None

    llm_data: Optional[LlmData] = None

    tool_calls: Optional[List[ToolCall]] = None

    translation_data: Optional[TranslationData] = None

    web_search_data: Optional[WebSearchData] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from typing_extensions import Literal, TypedDict

__all__ = ["ToolWebSearchParams"]


class ToolWebSearchParams(TypedDict, total=False):
    chunks_per_source: int
    """Only applies when `search_depth` is `advanced`.

    Specifies how many text segments to extract from each source. Limited to 3
    chunks maximum.
    """

    country: Literal[
        "afghanistan",
        "albania",
        "algeria",
        "andorra",
        "angola",
        "argentina",
        "armenia",
        "australia",
        "austria",
        "azerbaijan",
        "bahamas",
        "bahrain",
        "bangladesh",
        "barbados",
        "belarus",
        "belgium",
        "belize",
        "benin",
        "bhutan",
        "bolivia",
        "bosnia and herzegovina",
        "botswana",
        "brazil",
        "brunei",
        "bulgaria",
        "burkina faso",
        "burundi",
        "cambodia",
        "cameroon",
        "canada",
        "cape verde",
        "central african republic",
        "chad",
        "chile",
        "china",
        "colombia",
        "comoros",
        "congo",
        "costa rica",
        "croatia",
        "cuba",
        "cyprus",
        "czech republic",
        "denmark",
        "djibouti",
        "dominican republic",
        "ecuador",
        "egypt",
        "el salvador",
        "equatorial guinea",
        "eritrea",
        "estonia",
        "ethiopia",
        "fiji",
        "finland",
        "france",
        "gabon",
        "gambia",
        "georgia",
        "germany",
        "ghana",
        "greece",
        "guatemala",
        "guinea",
        "haiti",
        "honduras",
        "hungary",
        "iceland",
        "india",
        "indonesia",
        "iran",
        "iraq",
        "ireland",
        "israel",
        "italy",
        "jamaica",
        "japan",
        "jordan",
        "kazakhstan",
        "kenya",
        "kuwait",
        "kyrgyzstan",
        "latvia",
        "lebanon",
        "lesotho",
        "liberia",
        "libya",
        "liechtenstein",
        "lithuania",
        "luxembourg",
        "madagascar",
        "malawi",
        "malaysia",
        "maldives",
        "mali",
        "malta",
        "mauritania",
        "mauritius",
        "mexico",
        "moldova",
        "monaco",
        "mongolia",
        "montenegro",
        "morocco",
        "mozambique",
        "myanmar",
        "namibia",
        "nepal",
        "netherlands",
        "new zealand",
        "nicaragua",
        "niger",
        "nigeria",
        "north korea",
        "north macedonia",
        "norway",
        "oman",
        "pakistan",
        "panama",
        "papua new guinea",
        "paraguay",
        "peru",
        "philippines",
        "poland",
        "portugal",
        "qatar",
        "romania",
        "russia",
        "rwanda",
        "saudi arabia",
        "senegal",
        "serbia",
        "singapore",
        "slovakia",
        "slovenia",
        "somalia",
        "south africa",
        "south korea",
        "south sudan",
        "spain",
        "sri lanka",
        "sudan",
        "sweden",
        "switzerland",
        "syria",
        "taiwan",
        "tajikistan",
        "tanzania",
        "thailand",
        "togo",
        "trinidad and tobago",
        "tunisia",
        "turkey",
        "turkmenistan",
        "uganda",
        "ukraine",
        "united arab emirates",
        "united kingdom",
        "united states",
        "uruguay",
        "uzbekistan",
        "venezuela",
        "vietnam",
        "yemen",
        "zambia",
        "zimbabwe",
    ]
    """Localizes search results to a specific country.

    Only applies to general topic searches.
    """

    days: int
    """For news topic searches, specifies how many days of news coverage to include."""

    exclude_domains: List[str]
    """Domains to exclude from the search. If unset, the search includes all domains."""

    include_answer: bool
    """Whether to include a generated answer to the query in the response.

    If `false`, only search results are returned.
    """

    include_domains: List[str]
    """Domains to include in the search. If unset, the search includes all domains."""

    include_raw_content: Union[Literal["text", "markdown"], bool]
    """Controls how raw content is included in search results:

    - `text`: Returns plain text without formatting markup
    - `markdown`: Returns structured content with markdown formatting (headers,
      links, bold text)
    - `true`: Same as `markdown`
    - `false`: Raw content is not included (default if unset)
    """

    max_results: int
    """Limits the number of search results returned. Cannot exceed 20 sources."""

    query: str
    """The search query."""

    search_depth: Literal["basic", "advanced"]
    """Controls search comprehensiveness:

    - `basic`: Returns fewer but highly relevant results
    - `advanced`: Performs a deeper search with more results
    """

    stream: bool
    """Enables streaming of search results as they become available."""

    time_range: Literal["day", "week", "month", "year", "d", "w", "m", "y"]
    """
    Filters results to content published within the specified time range back from
    the current date. For example, `week` or `w` returns results from the past 7
    days.
    """

    topic: Literal["general", "news"]
    """The search topic category.

    Use `news` for current events and news articles, or `general` for broader web
    search.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .function_definition import FunctionDefinition

__all__ = [
    "ToolParam",
    "FunctionTool",
    "GraphTool",
    "GraphToolFunction",
    "LlmTool",
    "LlmToolFunction",
    "TranslationTool",
    "TranslationToolFunction",
    "VisionTool",
    "VisionToolFunction",
    "VisionToolFunctionVariable",
    "WebSearchTool",
    "WebSearchToolFunction",
]


class FunctionTool(TypedDict, total=False):
    function: Required[FunctionDefinition]
    """A tool that uses a custom function."""

    type: Required[Literal["function"]]
    """The type of tool."""


class GraphToolFunction(TypedDict, total=False):
    graph_ids: Required[List[str]]
    """An array of graph IDs to use in the tool."""

    subqueries: Required[bool]
    """Boolean to indicate whether to include subqueries in the response."""

    description: str
    """A description of the graph content."""


class GraphTool(TypedDict, total=False):
    function: Required[GraphToolFunction]
    """A tool that uses Knowledge Graphs as context for responses."""

    type: Required[Literal["graph"]]
    """The type of tool."""


class LlmToolFunction(TypedDict, total=False):
    description: Required[str]
    """A description of the model to use."""

    model: Required[str]
    """The model to use."""


class LlmTool(TypedDict, total=False):
    function: Required[LlmToolFunction]
    """A tool that uses another Writer model to generate a response."""

    type: Required[Literal["llm"]]
    """The type of tool."""


class TranslationToolFunction(TypedDict, total=False):
    formality: Required[bool]
    """Whether to use formal or informal language in the translation.

    See the
    [list of languages that support formality](https://dev.writer.com/api-reference/translation-api/language-support#formality).
    If the language does not support formality, this parameter is ignored.
    """

    length_control: Required[bool]
    """Whether to control the length of the translated text.

    See the
    [list of languages that support length control](https://dev.writer.com/api-reference/translation-api/language-support#length-control).
    If the language does not support length control, this parameter is ignored.
    """

    mask_profanity: Required[bool]
    """Whether to mask profane words in the translated text.

    See the
    [list of languages that do not support profanity masking](https://dev.writer.com/api-reference/translation-api/language-support#profanity-masking).
    If the language does not support profanity masking, this parameter is ignored.
    """

    model: Required[Literal["palmyra-translate"]]
    """The model to use for translation."""

    source_language_code: str
    """Optional.

    The [ISO-639-1](https://en.wikipedia.org/wiki/List_of_ISO_639_language_codes)
    language code of the original text to translate. For example, `en` for English,
    `zh` for Chinese, `fr` for French, `es` for Spanish. If the language has a
    variant, the code appends the two-digit
    [ISO-3166 country code](https://en.wikipedia.org/wiki/List_of_ISO_3166_country_codes).
    If you do not provide a language code, the LLM detects the language of the text.
    """

    target_language_code: str
    """Optional.

    The [ISO-639-1](https://en.wikipedia.org/wiki/List_of_ISO_639_language_codes)
    language code of the target language for the translation. For example, `en` for
    English, `zh` for Chinese, `fr` for French, `es` for Spanish. If the language
    has a variant, the code appends the two-digit
    [ISO-3166 country code](https://en.wikipedia.org/wiki/List_of_ISO_3166_country_codes).
    If you do not provide a language code, the LLM uses the content of the chat
    message to determine the target language.
    """


class TranslationTool(TypedDict, total=False):
    function: Required[TranslationToolFunction]
    """A tool that uses Palmyra Translate to translate text."""

    type: Required[Literal["translation"]]
    """The type of tool."""


class VisionToolFunctionVariable(TypedDict, total=False):
    file_id: Required[str]
    """The File ID of the image to analyze.

    The file must be uploaded to the Writer platform before you use it with the
    Vision tool. The maximum allowed file size is 7MB.
    """

    name: Required[str]
    """The name of the file variable.

    You must reference this name in the `message.content` field of the request to
    the chat completions endpoint. Use double curly braces (`{{}}`) to reference the
    file. For example,
    `Describe the difference between the image {{image_1}} and the image {{image_2}}`.
    """


class VisionToolFunction(TypedDict, total=False):
    model: Required[Literal["palmyra-vision"]]
    """The model to use for image analysis."""

    variables: Required[Iterable[VisionToolFunctionVariable]]


class VisionTool(TypedDict, total=False):
    function: Required[VisionToolFunction]
    """A tool that uses Palmyra Vision to analyze images."""

    type: Required[Literal["vision"]]
    """The type of tool."""


class WebSearchToolFunction(TypedDict, total=False):
    exclude_domains: Required[List[str]]
    """An array of domains to exclude from the search results."""

    include_domains: Required[List[str]]
    """An array of domains to include in the search results."""


class WebSearchTool(TypedDict, total=False):
    function: Required[WebSearchToolFunction]
    """A tool that uses web search to find information."""

    type: Required[Literal["web_search"]]
    """The type of tool."""


ToolParam: TypeAlias = Union[FunctionTool, GraphTool, LlmTool, TranslationTool, VisionTool, WebSearchTool]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["GraphUpdateParams", "URL"]


class GraphUpdateParams(TypedDict, total=False):
    description: str
    """A description of the Knowledge Graph (max 255 characters).

    Omitting this field leaves the description unchanged.
    """

    name: str
    """The name of the Knowledge Graph (max 255 characters).

    Omitting this field leaves the name unchanged.
    """

    urls: Iterable[URL]
    """An array of web connector URLs to update for this Knowledge Graph.

    You can only connect URLs to Knowledge Graphs with the type `web`. To clear the
    list of URLs, set this field to an empty array.
    """


class URL(TypedDict, total=False):
    type: Required[Literal["single_page", "sub_pages"]]
    """The type of web connector processing for this URL."""

    url: Required[str]
    """The URL to be processed by the web connector."""

    exclude_urls: List[str]
    """An array of URLs to exclude from processing within this web connector."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["GraphUpdateResponse", "URL", "URLStatus"]


class URLStatus(BaseModel):
    status: Literal["validating", "success", "error"]
    """The current status of the URL processing."""

    error_type: Optional[
        Literal["invalid_url", "not_searchable", "not_found", "paywall_or_login_page", "unexpected_error"]
    ] = None
    """The type of error that occurred during processing, if any."""


class URL(BaseModel):
    status: URLStatus
    """The current status of the URL processing."""

    type: Literal["single_page", "sub_pages"]
    """The type of web connector processing for this URL."""

    url: str
    """The URL to be processed by the web connector."""

    exclude_urls: Optional[List[str]] = None
    """An array of URLs to exclude from processing within this web connector."""


class GraphUpdateResponse(BaseModel):
    id: str
    """A unique identifier of the Knowledge Graph."""

    created_at: datetime
    """The timestamp when the Knowledge Graph was created."""

    name: str
    """The name of the Knowledge Graph (max 255 characters)."""

    description: Optional[str] = None
    """A description of the Knowledge Graph (max 255 characters)."""

    urls: Optional[List[URL]] = None
    """An array of web connector URLs associated with this Knowledge Graph."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["Source"]


class Source(BaseModel):
    file_id: str
    """The unique identifier of the file."""

    snippet: str
    """A snippet of text from the source file."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ToolAIDetectResponse"]


class ToolAIDetectResponse(BaseModel):
    label: Literal["fake", "real"]

    score: float



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.applications import graph_update_params
from ...types.applications.application_graphs_response import ApplicationGraphsResponse

__all__ = ["GraphsResource", "AsyncGraphsResource"]


class GraphsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> GraphsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/writer/writer-python#accessing-raw-response-data-eg-headers
        """
        return GraphsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GraphsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/writer/writer-python#with_streaming_response
        """
        return GraphsResourceWithStreamingResponse(self)

    def update(
        self,
        application_id: str,
        *,
        graph_ids: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationGraphsResponse:
        """
        Updates the list of Knowledge Graphs associated with a no-code chat agent.

        Args:
          graph_ids: A list of Knowledge Graph IDs to associate with the application. Note that this
              will replace the existing list of Knowledge Graphs associated with the
              application, not add to it.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return self._put(
            f"/v1/applications/{application_id}/graphs",
            body=maybe_transform({"graph_ids": graph_ids}, graph_update_params.GraphUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationGraphsResponse,
        )

    def list(
        self,
        application_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationGraphsResponse:
        """
        Retrieve Knowledge Graphs associated with a no-code agent that has chat
        capabilities.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return self._get(
            f"/v1/applications/{application_id}/graphs",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationGraphsResponse,
        )


class AsyncGraphsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncGraphsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/writer/writer-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGraphsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGraphsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/writer/writer-python#with_streaming_response
        """
        return AsyncGraphsResourceWithStreamingResponse(self)

    async def update(
        self,
        application_id: str,
        *,
        graph_ids: List[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationGraphsResponse:
        """
        Updates the list of Knowledge Graphs associated with a no-code chat agent.

        Args:
          graph_ids: A list of Knowledge Graph IDs to associate with the application. Note that this
              will replace the existing list of Knowledge Graphs associated with the
              application, not add to it.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return await self._put(
            f"/v1/applications/{application_id}/graphs",
            body=await async_maybe_transform({"graph_ids": graph_ids}, graph_update_params.GraphUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationGraphsResponse,
        )

    async def list(
        self,
        application_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationGraphsResponse:
        """
        Retrieve Knowledge Graphs associated with a no-code agent that has chat
        capabilities.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return await self._get(
            f"/v1/applications/{application_id}/graphs",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationGraphsResponse,
        )


class GraphsResourceWithRawResponse:
    def __init__(self, graphs: GraphsResource) -> None:
        self._graphs = graphs

        self.update = to_raw_response_wrapper(
            graphs.update,
        )
        self.list = to_raw_response_wrapper(
            graphs.list,
        )


class AsyncGraphsResourceWithRawResponse:
    def __init__(self, graphs: AsyncGraphsResource) -> None:
        self._graphs = graphs

        self.update = async_to_raw_response_wrapper(
            graphs.update,
        )
        self.list = async_to_raw_response_wrapper(
            graphs.list,
        )


class GraphsResourceWithStreamingResponse:
    def __init__(self, graphs: GraphsResource) -> None:
        self._graphs = graphs

        self.update = to_streamed_response_wrapper(
            graphs.update,
        )
        self.list = to_streamed_response_wrapper(
            graphs.list,
        )


class AsyncGraphsResourceWithStreamingResponse:
    def __init__(self, graphs: AsyncGraphsResource) -> None:
        self._graphs = graphs

        self.update = async_to_streamed_response_wrapper(
            graphs.update,
        )
        self.list = async_to_streamed_response_wrapper(
            graphs.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Union, Mapping
from typing_extensions import Self, override

import httpx

from writerai import _exceptions

from ._qs import Querystring
from ._types import (
    NOT_GIVEN,
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._utils import is_given, get_async_library
from ._version import __version__
from .resources import chat, files, graphs, models, vision, completions, translation
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import WriterError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)
from .resources.tools import tools
from .resources.applications import applications

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "Writer", "AsyncWriter", "Client", "AsyncClient"]


_SDK_NAMESPACE_PREFIX = "WRITER_SDK_"
_SDK_HEADER_PREFIX = _SDK_NAMESPACE_PREFIX + "HEADER_"


def _extract_sdk_env_headers() -> dict[str, str]:
    """
    Collect headers defined through environment variables of the form
    `WRITER_SDK_HEADER_<HEADER_NAME>`.

    Example
    --------
    >>> os.environ["WRITER_SDK_HEADER_X_FOO_BAR"] = "abc123"
    >>> _extract_sdk_env_headers()
    {'X-Foo-Bar': 'abc123'}
    """
    headers: dict[str, str] = {}

    for key, value in os.environ.items():
        if not key.startswith(_SDK_HEADER_PREFIX):
            continue

        # Strip the prefix and convert
        raw = key[len(_SDK_HEADER_PREFIX):]
        parts = raw.split("_")
        canonical = "-".join(
            word.capitalize() if len(word) > 1 else word.upper()
            for word in parts
        )
        headers[canonical] = value

    return headers


class Writer(SyncAPIClient):
    applications: applications.ApplicationsResource
    chat: chat.ChatResource
    completions: completions.CompletionsResource
    models: models.ModelsResource
    graphs: graphs.GraphsResource
    files: files.FilesResource
    tools: tools.ToolsResource
    translation: translation.TranslationResource
    vision: vision.VisionResource
    with_raw_response: WriterWithRawResponse
    with_streaming_response: WriterWithStreamedResponse

    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Writer client instance.

        This automatically infers the `api_key` argument from the `WRITER_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("WRITER_API_KEY")
        if api_key is None:
            raise WriterError(
                "The api_key client option must be set either by passing api_key to the client or by setting the WRITER_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("WRITER_BASE_URL")
        if base_url is None:
            base_url = f"https://api.writer.com"

        env_headers = _extract_sdk_env_headers()
        default_headers = {**env_headers, **(default_headers or {})}

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._default_stream_cls = Stream

        self.applications = applications.ApplicationsResource(self)
        self.chat = chat.ChatResource(self)
        self.completions = completions.CompletionsResource(self)
        self.models = models.ModelsResource(self)
        self.graphs = graphs.GraphsResource(self)
        self.files = files.FilesResource(self)
        self.tools = tools.ToolsResource(self)
        self.translation = translation.TranslationResource(self)
        self.vision = vision.VisionResource(self)
        self.with_raw_response = WriterWithRawResponse(self)
        self.with_streaming_response = WriterWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncWriter(AsyncAPIClient):
    applications: applications.AsyncApplicationsResource
    chat: chat.AsyncChatResource
    completions: completions.AsyncCompletionsResource
    models: models.AsyncModelsResource
    graphs: graphs.AsyncGraphsResource
    files: files.AsyncFilesResource
    tools: tools.AsyncToolsResource
    translation: translation.AsyncTranslationResource
    vision: vision.AsyncVisionResource
    with_raw_response: AsyncWriterWithRawResponse
    with_streaming_response: AsyncWriterWithStreamedResponse

    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncWriter client instance.

        This automatically infers the `api_key` argument from the `WRITER_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("WRITER_API_KEY")
        if api_key is None:
            raise WriterError(
                "The api_key client option must be set either by passing api_key to the client or by setting the WRITER_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("WRITER_BASE_URL")
        if base_url is None:
            base_url = f"https://api.writer.com"

        env_headers = _extract_sdk_env_headers()
        merged_headers = {**env_headers, **(default_headers or {})}

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=merged_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._default_stream_cls = AsyncStream

        self.applications = applications.AsyncApplicationsResource(self)
        self.chat = chat.AsyncChatResource(self)
        self.completions = completions.AsyncCompletionsResource(self)
        self.models = models.AsyncModelsResource(self)
        self.graphs = graphs.AsyncGraphsResource(self)
        self.files = files.AsyncFilesResource(self)
        self.tools = tools.AsyncToolsResource(self)
        self.translation = translation.AsyncTranslationResource(self)
        self.vision = vision.AsyncVisionResource(self)
        self.with_raw_response = AsyncWriterWithRawResponse(self)
        self.with_streaming_response = AsyncWriterWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class WriterWithRawResponse:
    def __init__(self, client: Writer) -> None:
        self.applications = applications.ApplicationsResourceWithRawResponse(client.applications)
        self.chat = chat.ChatResourceWithRawResponse(client.chat)
        self.completions = completions.CompletionsResourceWithRawResponse(client.completions)
        self.models = models.ModelsResourceWithRawResponse(client.models)
        self.graphs = graphs.GraphsResourceWithRawResponse(client.graphs)
        self.files = files.FilesResourceWithRawResponse(client.files)
        self.tools = tools.ToolsResourceWithRawResponse(client.tools)
        self.translation = translation.TranslationResourceWithRawResponse(client.translation)
        self.vision = vision.VisionResourceWithRawResponse(client.vision)


class AsyncWriterWithRawResponse:
    def __init__(self, client: AsyncWriter) -> None:
        self.applications = applications.AsyncApplicationsResourceWithRawResponse(client.applications)
        self.chat = chat.AsyncChatResourceWithRawResponse(client.chat)
        self.completions = completions.AsyncCompletionsResourceWithRawResponse(client.completions)
        self.models = models.AsyncModelsResourceWithRawResponse(client.models)
        self.graphs = graphs.AsyncGraphsResourceWithRawResponse(client.graphs)
        self.files = files.AsyncFilesResourceWithRawResponse(client.files)
        self.tools = tools.AsyncToolsResourceWithRawResponse(client.tools)
        self.translation = translation.AsyncTranslationResourceWithRawResponse(client.translation)
        self.vision = vision.AsyncVisionResourceWithRawResponse(client.vision)


class WriterWithStreamedResponse:
    def __init__(self, client: Writer) -> None:
        self.applications = applications.ApplicationsResourceWithStreamingResponse(client.applications)
        self.chat = chat.ChatResourceWithStreamingResponse(client.chat)
        self.completions = completions.CompletionsResourceWithStreamingResponse(client.completions)
        self.models = models.ModelsResourceWithStreamingResponse(client.models)
        self.graphs = graphs.GraphsResourceWithStreamingResponse(client.graphs)
        self.files = files.FilesResourceWithStreamingResponse(client.files)
        self.tools = tools.ToolsResourceWithStreamingResponse(client.tools)
        self.translation = translation.TranslationResourceWithStreamingResponse(client.translation)
        self.vision = vision.VisionResourceWithStreamingResponse(client.vision)


class AsyncWriterWithStreamedResponse:
    def __init__(self, client: AsyncWriter) -> None:
        self.applications = applications.AsyncApplicationsResourceWithStreamingResponse(client.applications)
        self.chat = chat.AsyncChatResourceWithStreamingResponse(client.chat)
        self.completions = completions.AsyncCompletionsResourceWithStreamingResponse(client.completions)
        self.models = models.AsyncModelsResourceWithStreamingResponse(client.models)
        self.graphs = graphs.AsyncGraphsResourceWithStreamingResponse(client.graphs)
        self.files = files.AsyncFilesResourceWithStreamingResponse(client.files)
        self.tools = tools.AsyncToolsResourceWithStreamingResponse(client.tools)
        self.translation = translation.AsyncTranslationResourceWithStreamingResponse(client.translation)
        self.vision = vision.AsyncVisionResourceWithStreamingResponse(client.vision)


Client = Writer

AsyncClient = AsyncWriter



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from typing_extensions import Literal

import httpx

from ...types import (
    tool_ai_detect_params,
    tool_parse_pdf_params,
    tool_web_search_params,
    tool_context_aware_splitting_params,
)
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .comprehend import (
    ComprehendResource,
    AsyncComprehendResource,
    ComprehendResourceWithRawResponse,
    AsyncComprehendResourceWithRawResponse,
    ComprehendResourceWithStreamingResponse,
    AsyncComprehendResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.tool_ai_detect_response import ToolAIDetectResponse
from ...types.tool_parse_pdf_response import ToolParsePdfResponse
from ...types.tool_web_search_response import ToolWebSearchResponse
from ...types.tool_context_aware_splitting_response import ToolContextAwareSplittingResponse

__all__ = ["ToolsResource", "AsyncToolsResource"]


class ToolsResource(SyncAPIResource):
    @cached_property
    def comprehend(self) -> ComprehendResource:
        return ComprehendResource(self._client)

    @cached_property
    def with_raw_response(self) -> ToolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/writer/writer-python#accessing-raw-response-data-eg-headers
        """
        return ToolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ToolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/writer/writer-python#with_streaming_response
        """
        return ToolsResourceWithStreamingResponse(self)

    def ai_detect(
        self,
        *,
        input: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ToolAIDetectResponse:
        """Detects if content is AI- or human-generated, with a confidence score.

        Content
        must have at least 350 characters

        Args:
          input: The content to determine if it is AI- or human-generated. Content must have at
              least 350 characters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/tools/ai-detect",
            body=maybe_transform({"input": input}, tool_ai_detect_params.ToolAIDetectParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolAIDetectResponse,
        )

    def context_aware_splitting(
        self,
        *,
        strategy: Literal["llm_split", "fast_split", "hybrid_split"],
        text: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ToolContextAwareSplittingResponse:
        """
        Splits a long block of text (maximum 4000 words) into smaller chunks while
        preserving the semantic meaning of the text and context between the chunks.

        Args:
          strategy: The strategy to use for splitting the text into chunks. `llm_split` uses the
              language model to split the text, `fast_split` uses a fast heuristic-based
              approach, and `hybrid_split` combines both strategies.

          text: The text to split into chunks.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/tools/context-aware-splitting",
            body=maybe_transform(
                {
                    "strategy": strategy,
                    "text": text,
                },
                tool_context_aware_splitting_params.ToolContextAwareSplittingParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolContextAwareSplittingResponse,
        )

    def parse_pdf(
        self,
        file_id: str,
        *,
        format: Literal["text", "markdown"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ToolParsePdfResponse:
        """
        Parse PDF to other formats.

        Args:
          format: The format into which the PDF content should be converted.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not file_id:
            raise ValueError(f"Expected a non-empty value for `file_id` but received {file_id!r}")
        return self._post(
            f"/v1/tools/pdf-parser/{file_id}",
            body=maybe_transform({"format": format}, tool_parse_pdf_params.ToolParsePdfParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolParsePdfResponse,
        )

    def web_search(
        self,
        *,
        chunks_per_source: int | NotGiven = NOT_GIVEN,
        country: Literal[
            "afghanistan",
            "albania",
            "algeria",
            "andorra",
            "angola",
            "argentina",
            "armenia",
            "australia",
            "austria",
            "azerbaijan",
            "bahamas",
            "bahrain",
            "bangladesh",
            "barbados",
            "belarus",
            "belgium",
            "belize",
            "benin",
            "bhutan",
            "bolivia",
            "bosnia and herzegovina",
            "botswana",
            "brazil",
            "brunei",
            "bulgaria",
            "burkina faso",
            "burundi",
            "cambodia",
            "cameroon",
            "canada",
            "cape verde",
            "central african republic",
            "chad",
            "chile",
            "china",
            "colombia",
            "comoros",
            "congo",
            "costa rica",
            "croatia",
            "cuba",
            "cyprus",
            "czech republic",
            "denmark",
            "djibouti",
            "dominican republic",
            "ecuador",
            "egypt",
            "el salvador",
            "equatorial guinea",
            "eritrea",
            "estonia",
            "ethiopia",
            "fiji",
            "finland",
            "france",
            "gabon",
            "gambia",
            "georgia",
            "germany",
            "ghana",
            "greece",
            "guatemala",
            "guinea",
            "haiti",
            "honduras",
            "hungary",
            "iceland",
            "india",
            "indonesia",
            "iran",
            "iraq",
            "ireland",
            "israel",
            "italy",
            "jamaica",
            "japan",
            "jordan",
            "kazakhstan",
            "kenya",
            "kuwait",
            "kyrgyzstan",
            "latvia",
            "lebanon",
            "lesotho",
            "liberia",
            "libya",
            "liechtenstein",
            "lithuania",
            "luxembourg",
            "madagascar",
            "malawi",
            "malaysia",
            "maldives",
            "mali",
            "malta",
            "mauritania",
            "mauritius",
            "mexico",
            "moldova",
            "monaco",
            "mongolia",
            "montenegro",
            "morocco",
            "mozambique",
            "myanmar",
            "namibia",
            "nepal",
            "netherlands",
            "new zealand",
            "nicaragua",
            "niger",
            "nigeria",
            "north korea",
            "north macedonia",
            "norway",
            "oman",
            "pakistan",
            "panama",
            "papua new guinea",
            "paraguay",
            "peru",
            "philippines",
            "poland",
            "portugal",
            "qatar",
            "romania",
            "russia",
            "rwanda",
            "saudi arabia",
            "senegal",
            "serbia",
            "singapore",
            "slovakia",
            "slovenia",
            "somalia",
            "south africa",
            "south korea",
            "south sudan",
            "spain",
            "sri lanka",
            "sudan",
            "sweden",
            "switzerland",
            "syria",
            "taiwan",
            "tajikistan",
            "tanzania",
            "thailand",
            "togo",
            "trinidad and tobago",
            "tunisia",
            "turkey",
            "turkmenistan",
            "uganda",
            "ukraine",
            "united arab emirates",
            "united kingdom",
            "united states",
            "uruguay",
            "uzbekistan",
            "venezuela",
            "vietnam",
            "yemen",
            "zambia",
            "zimbabwe",
        ]
        | NotGiven = NOT_GIVEN,
        days: int | NotGiven = NOT_GIVEN,
        exclude_domains: List[str] | NotGiven = NOT_GIVEN,
        include_answer: bool | NotGiven = NOT_GIVEN,
        include_domains: List[str] | NotGiven = NOT_GIVEN,
        include_raw_content: Union[Literal["text", "markdown"], bool] | NotGiven = NOT_GIVEN,
        max_results: int | NotGiven = NOT_GIVEN,
        query: str | NotGiven = NOT_GIVEN,
        search_depth: Literal["basic", "advanced"] | NotGiven = NOT_GIVEN,
        stream: bool | NotGiven = NOT_GIVEN,
        time_range: Literal["day", "week", "month", "year", "d", "w", "m", "y"] | NotGiven = NOT_GIVEN,
        topic: Literal["general", "news"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ToolWebSearchResponse:
        """
        Search the web for information about a given query and return relevant results
        with source URLs.

        Args:
          chunks_per_source: Only applies when `search_depth` is `advanced`. Specifies how many text segments
              to extract from each source. Limited to 3 chunks maximum.

          country: Localizes search results to a specific country. Only applies to general topic
              searches.

          days: For news topic searches, specifies how many days of news coverage to include.

          exclude_domains: Domains to exclude from the search. If unset, the search includes all domains.

          include_answer: Whether to include a generated answer to the query in the response. If `false`,
              only search results are returned.

          include_domains: Domains to include in the search. If unset, the search includes all domains.

          include_raw_content:
              Controls how raw content is included in search results:

              - `text`: Returns plain text without formatting markup
              - `markdown`: Returns structured content with markdown formatting (headers,
                links, bold text)
              - `true`: Same as `markdown`
              - `false`: Raw content is not included (default if unset)

          max_results: Limits the number of search results returned. Cannot exceed 20 sources.

          query: The search query.

          search_depth:
              Controls search comprehensiveness:

              - `basic`: Returns fewer but highly relevant results
              - `advanced`: Performs a deeper search with more results

          stream: Enables streaming of search results as they become available.

          time_range: Filters results to content published within the specified time range back from
              the current date. For example, `week` or `w` returns results from the past 7
              days.

          topic: The search topic category. Use `news` for current events and news articles, or
              `general` for broader web search.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/tools/web-search",
            body=maybe_transform(
                {
                    "chunks_per_source": chunks_per_source,
                    "country": country,
                    "days": days,
                    "exclude_domains": exclude_domains,
                    "include_answer": include_answer,
                    "include_domains": include_domains,
                    "include_raw_content": include_raw_content,
                    "max_results": max_results,
                    "query": query,
                    "search_depth": search_depth,
                    "stream": stream,
                    "time_range": time_range,
                    "topic": topic,
                },
                tool_web_search_params.ToolWebSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolWebSearchResponse,
        )


class AsyncToolsResource(AsyncAPIResource):
    @cached_property
    def comprehend(self) -> AsyncComprehendResource:
        return AsyncComprehendResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncToolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/writer/writer-python#accessing-raw-response-data-eg-headers
        """
        return AsyncToolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncToolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/writer/writer-python#with_streaming_response
        """
        return AsyncToolsResourceWithStreamingResponse(self)

    async def ai_detect(
        self,
        *,
        input: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ToolAIDetectResponse:
        """Detects if content is AI- or human-generated, with a confidence score.

        Content
        must have at least 350 characters

        Args:
          input: The content to determine if it is AI- or human-generated. Content must have at
              least 350 characters.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/tools/ai-detect",
            body=await async_maybe_transform({"input": input}, tool_ai_detect_params.ToolAIDetectParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolAIDetectResponse,
        )

    async def context_aware_splitting(
        self,
        *,
        strategy: Literal["llm_split", "fast_split", "hybrid_split"],
        text: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ToolContextAwareSplittingResponse:
        """
        Splits a long block of text (maximum 4000 words) into smaller chunks while
        preserving the semantic meaning of the text and context between the chunks.

        Args:
          strategy: The strategy to use for splitting the text into chunks. `llm_split` uses the
              language model to split the text, `fast_split` uses a fast heuristic-based
              approach, and `hybrid_split` combines both strategies.

          text: The text to split into chunks.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/tools/context-aware-splitting",
            body=await async_maybe_transform(
                {
                    "strategy": strategy,
                    "text": text,
                },
                tool_context_aware_splitting_params.ToolContextAwareSplittingParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolContextAwareSplittingResponse,
        )

    async def parse_pdf(
        self,
        file_id: str,
        *,
        format: Literal["text", "markdown"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ToolParsePdfResponse:
        """
        Parse PDF to other formats.

        Args:
          format: The format into which the PDF content should be converted.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not file_id:
            raise ValueError(f"Expected a non-empty value for `file_id` but received {file_id!r}")
        return await self._post(
            f"/v1/tools/pdf-parser/{file_id}",
            body=await async_maybe_transform({"format": format}, tool_parse_pdf_params.ToolParsePdfParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolParsePdfResponse,
        )

    async def web_search(
        self,
        *,
        chunks_per_source: int | NotGiven = NOT_GIVEN,
        country: Literal[
            "afghanistan",
            "albania",
            "algeria",
            "andorra",
            "angola",
            "argentina",
            "armenia",
            "australia",
            "austria",
            "azerbaijan",
            "bahamas",
            "bahrain",
            "bangladesh",
            "barbados",
            "belarus",
            "belgium",
            "belize",
            "benin",
            "bhutan",
            "bolivia",
            "bosnia and herzegovina",
            "botswana",
            "brazil",
            "brunei",
            "bulgaria",
            "burkina faso",
            "burundi",
            "cambodia",
            "cameroon",
            "canada",
            "cape verde",
            "central african republic",
            "chad",
            "chile",
            "china",
            "colombia",
            "comoros",
            "congo",
            "costa rica",
            "croatia",
            "cuba",
            "cyprus",
            "czech republic",
            "denmark",
            "djibouti",
            "dominican republic",
            "ecuador",
            "egypt",
            "el salvador",
            "equatorial guinea",
            "eritrea",
            "estonia",
            "ethiopia",
            "fiji",
            "finland",
            "france",
            "gabon",
            "gambia",
            "georgia",
            "germany",
            "ghana",
            "greece",
            "guatemala",
            "guinea",
            "haiti",
            "honduras",
            "hungary",
            "iceland",
            "india",
            "indonesia",
            "iran",
            "iraq",
            "ireland",
            "israel",
            "italy",
            "jamaica",
            "japan",
            "jordan",
            "kazakhstan",
            "kenya",
            "kuwait",
            "kyrgyzstan",
            "latvia",
            "lebanon",
            "lesotho",
            "liberia",
            "libya",
            "liechtenstein",
            "lithuania",
            "luxembourg",
            "madagascar",
            "malawi",
            "malaysia",
            "maldives",
            "mali",
            "malta",
            "mauritania",
            "mauritius",
            "mexico",
            "moldova",
            "monaco",
            "mongolia",
            "montenegro",
            "morocco",
            "mozambique",
            "myanmar",
            "namibia",
            "nepal",
            "netherlands",
            "new zealand",
            "nicaragua",
            "niger",
            "nigeria",
            "north korea",
            "north macedonia",
            "norway",
            "oman",
            "pakistan",
            "panama",
            "papua new guinea",
            "paraguay",
            "peru",
            "philippines",
            "poland",
            "portugal",
            "qatar",
            "romania",
            "russia",
            "rwanda",
            "saudi arabia",
            "senegal",
            "serbia",
            "singapore",
            "slovakia",
            "slovenia",
            "somalia",
            "south africa",
            "south korea",
            "south sudan",
            "spain",
            "sri lanka",
            "sudan",
            "sweden",
            "switzerland",
            "syria",
            "taiwan",
            "tajikistan",
            "tanzania",
            "thailand",
            "togo",
            "trinidad and tobago",
            "tunisia",
            "turkey",
            "turkmenistan",
            "uganda",
            "ukraine",
            "united arab emirates",
            "united kingdom",
            "united states",
            "uruguay",
            "uzbekistan",
            "venezuela",
            "vietnam",
            "yemen",
            "zambia",
            "zimbabwe",
        ]
        | NotGiven = NOT_GIVEN,
        days: int | NotGiven = NOT_GIVEN,
        exclude_domains: List[str] | NotGiven = NOT_GIVEN,
        include_answer: bool | NotGiven = NOT_GIVEN,
        include_domains: List[str] | NotGiven = NOT_GIVEN,
        include_raw_content: Union[Literal["text", "markdown"], bool] | NotGiven = NOT_GIVEN,
        max_results: int | NotGiven = NOT_GIVEN,
        query: str | NotGiven = NOT_GIVEN,
        search_depth: Literal["basic", "advanced"] | NotGiven = NOT_GIVEN,
        stream: bool | NotGiven = NOT_GIVEN,
        time_range: Literal["day", "week", "month", "year", "d", "w", "m", "y"] | NotGiven = NOT_GIVEN,
        topic: Literal["general", "news"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ToolWebSearchResponse:
        """
        Search the web for information about a given query and return relevant results
        with source URLs.

        Args:
          chunks_per_source: Only applies when `search_depth` is `advanced`. Specifies how many text segments
              to extract from each source. Limited to 3 chunks maximum.

          country: Localizes search results to a specific country. Only applies to general topic
              searches.

          days: For news topic searches, specifies how many days of news coverage to include.

          exclude_domains: Domains to exclude from the search. If unset, the search includes all domains.

          include_answer: Whether to include a generated answer to the query in the response. If `false`,
              only search results are returned.

          include_domains: Domains to include in the search. If unset, the search includes all domains.

          include_raw_content:
              Controls how raw content is included in search results:

              - `text`: Returns plain text without formatting markup
              - `markdown`: Returns structured content with markdown formatting (headers,
                links, bold text)
              - `true`: Same as `markdown`
              - `false`: Raw content is not included (default if unset)

          max_results: Limits the number of search results returned. Cannot exceed 20 sources.

          query: The search query.

          search_depth:
              Controls search comprehensiveness:

              - `basic`: Returns fewer but highly relevant results
              - `advanced`: Performs a deeper search with more results

          stream: Enables streaming of search results as they become available.

          time_range: Filters results to content published within the specified time range back from
              the current date. For example, `week` or `w` returns results from the past 7
              days.

          topic: The search topic category. Use `news` for current events and news articles, or
              `general` for broader web search.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/tools/web-search",
            body=await async_maybe_transform(
                {
                    "chunks_per_source": chunks_per_source,
                    "country": country,
                    "days": days,
                    "exclude_domains": exclude_domains,
                    "include_answer": include_answer,
                    "include_domains": include_domains,
                    "include_raw_content": include_raw_content,
                    "max_results": max_results,
                    "query": query,
                    "search_depth": search_depth,
                    "stream": stream,
                    "time_range": time_range,
                    "topic": topic,
                },
                tool_web_search_params.ToolWebSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolWebSearchResponse,
        )


class ToolsResourceWithRawResponse:
    def __init__(self, tools: ToolsResource) -> None:
        self._tools = tools

        self.ai_detect = to_raw_response_wrapper(
            tools.ai_detect,
        )
        self.context_aware_splitting = to_raw_response_wrapper(
            tools.context_aware_splitting,
        )
        self.parse_pdf = to_raw_response_wrapper(
            tools.parse_pdf,
        )
        self.web_search = to_raw_response_wrapper(
            tools.web_search,
        )

    @cached_property
    def comprehend(self) -> ComprehendResourceWithRawResponse:
        return ComprehendResourceWithRawResponse(self._tools.comprehend)


class AsyncToolsResourceWithRawResponse:
    def __init__(self, tools: AsyncToolsResource) -> None:
        self._tools = tools

        self.ai_detect = async_to_raw_response_wrapper(
            tools.ai_detect,
        )
        self.context_aware_splitting = async_to_raw_response_wrapper(
            tools.context_aware_splitting,
        )
        self.parse_pdf = async_to_raw_response_wrapper(
            tools.parse_pdf,
        )
        self.web_search = async_to_raw_response_wrapper(
            tools.web_search,
        )

    @cached_property
    def comprehend(self) -> AsyncComprehendResourceWithRawResponse:
        return AsyncComprehendResourceWithRawResponse(self._tools.comprehend)


class ToolsResourceWithStreamingResponse:
    def __init__(self, tools: ToolsResource) -> None:
        self._tools = tools

        self.ai_detect = to_streamed_response_wrapper(
            tools.ai_detect,
        )
        self.context_aware_splitting = to_streamed_response_wrapper(
            tools.context_aware_splitting,
        )
        self.parse_pdf = to_streamed_response_wrapper(
            tools.parse_pdf,
        )
        self.web_search = to_streamed_response_wrapper(
            tools.web_search,
        )

    @cached_property
    def comprehend(self) -> ComprehendResourceWithStreamingResponse:
        return ComprehendResourceWithStreamingResponse(self._tools.comprehend)


class AsyncToolsResourceWithStreamingResponse:
    def __init__(self, tools: AsyncToolsResource) -> None:
        self._tools = tools

        self.ai_detect = async_to_streamed_response_wrapper(
            tools.ai_detect,
        )
        self.context_aware_splitting = async_to_streamed_response_wrapper(
            tools.context_aware_splitting,
        )
        self.parse_pdf = async_to_streamed_response_wrapper(
            tools.parse_pdf,
        )
        self.web_search = async_to_streamed_response_wrapper(
            tools.web_search,
        )

    @cached_property
    def comprehend(self) -> AsyncComprehendResourceWithStreamingResponse:
        return AsyncComprehendResourceWithStreamingResponse(self._tools.comprehend)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from writerai import Writer, AsyncWriter
from tests.utils import assert_matches_type
from writerai.types.applications import ApplicationGraphsResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestGraphs:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_update(self, client: Writer) -> None:
        graph = client.applications.graphs.update(
            application_id="application_id",
            graph_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(ApplicationGraphsResponse, graph, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Writer) -> None:
        response = client.applications.graphs.with_raw_response.update(
            application_id="application_id",
            graph_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        graph = response.parse()
        assert_matches_type(ApplicationGraphsResponse, graph, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Writer) -> None:
        with client.applications.graphs.with_streaming_response.update(
            application_id="application_id",
            graph_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            graph = response.parse()
            assert_matches_type(ApplicationGraphsResponse, graph, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Writer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `application_id` but received ''"):
            client.applications.graphs.with_raw_response.update(
                application_id="",
                graph_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            )

    @parametrize
    def test_method_list(self, client: Writer) -> None:
        graph = client.applications.graphs.list(
            "application_id",
        )
        assert_matches_type(ApplicationGraphsResponse, graph, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Writer) -> None:
        response = client.applications.graphs.with_raw_response.list(
            "application_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        graph = response.parse()
        assert_matches_type(ApplicationGraphsResponse, graph, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Writer) -> None:
        with client.applications.graphs.with_streaming_response.list(
            "application_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            graph = response.parse()
            assert_matches_type(ApplicationGraphsResponse, graph, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: Writer) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `application_id` but received ''"):
            client.applications.graphs.with_raw_response.list(
                "",
            )


class TestAsyncGraphs:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_update(self, async_client: AsyncWriter) -> None:
        graph = await async_client.applications.graphs.update(
            application_id="application_id",
            graph_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(ApplicationGraphsResponse, graph, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncWriter) -> None:
        response = await async_client.applications.graphs.with_raw_response.update(
            application_id="application_id",
            graph_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        graph = await response.parse()
        assert_matches_type(ApplicationGraphsResponse, graph, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncWriter) -> None:
        async with async_client.applications.graphs.with_streaming_response.update(
            application_id="application_id",
            graph_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            graph = await response.parse()
            assert_matches_type(ApplicationGraphsResponse, graph, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncWriter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `application_id` but received ''"):
            await async_client.applications.graphs.with_raw_response.update(
                application_id="",
                graph_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncWriter) -> None:
        graph = await async_client.applications.graphs.list(
            "application_id",
        )
        assert_matches_type(ApplicationGraphsResponse, graph, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncWriter) -> None:
        response = await async_client.applications.graphs.with_raw_response.list(
            "application_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        graph = await response.parse()
        assert_matches_type(ApplicationGraphsResponse, graph, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncWriter) -> None:
        async with async_client.applications.graphs.with_streaming_response.list(
            "application_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            graph = await response.parse()
            assert_matches_type(ApplicationGraphsResponse, graph, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncWriter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `application_id` but received ''"):
            await async_client.applications.graphs.with_raw_response.list(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ToolContextAwareSplittingParams"]


class ToolContextAwareSplittingParams(TypedDict, total=False):
    strategy: Required[Literal["llm_split", "fast_split", "hybrid_split"]]
    """The strategy to use for splitting the text into chunks.

    `llm_split` uses the language model to split the text, `fast_split` uses a fast
    heuristic-based approach, and `hybrid_split` combines both strategies.
    """

    text: Required[str]
    """The text to split into chunks."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import translation_translate_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.translation_response import TranslationResponse

__all__ = ["TranslationResource", "AsyncTranslationResource"]


class TranslationResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TranslationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/writer/writer-python#accessing-raw-response-data-eg-headers
        """
        return TranslationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TranslationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/writer/writer-python#with_streaming_response
        """
        return TranslationResourceWithStreamingResponse(self)

    def translate(
        self,
        *,
        formality: bool,
        length_control: bool,
        mask_profanity: bool,
        model: Literal["palmyra-translate"],
        source_language_code: str,
        target_language_code: str,
        text: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TranslationResponse:
        """
        Translate text from one language to another.

        Args:
          formality: Whether to use formal or informal language in the translation. See the
              [list of languages that support formality](https://dev.writer.com/api-reference/translation-api/language-support#formality).
              If the language does not support formality, this parameter is ignored.

          length_control: Whether to control the length of the translated text. See the
              [list of languages that support length control](https://dev.writer.com/api-reference/translation-api/language-support#length-control).
              If the language does not support length control, this parameter is ignored.

          mask_profanity: Whether to mask profane words in the translated text. See the
              [list of languages that do not support profanity masking](https://dev.writer.com/api-reference/translation-api/language-support#profanity-masking).
              If the language does not support profanity masking, this parameter is ignored.

          model: The model to use for translation.

          source_language_code: The [ISO-639-1](https://en.wikipedia.org/wiki/List_of_ISO_639_language_codes)
              language code of the original text to translate. For example, `en` for English,
              `zh` for Chinese, `fr` for French, `es` for Spanish. If the language has a
              variant, the code appends the two-digit
              [ISO-3166 country code](https://en.wikipedia.org/wiki/List_of_ISO_3166_country_codes).
              For example, Mexican Spanish is `es-MX`. See the
              [list of supported languages and language codes](https://dev.writer.com/api-reference/translation-api/language-support).

          target_language_code: The [ISO-639-1](https://en.wikipedia.org/wiki/List_of_ISO_639_language_codes)
              language code of the target language for the translation. For example, `en` for
              English, `zh` for Chinese, `fr` for French, `es` for Spanish. If the language
              has a variant, the code appends the two-digit
              [ISO-3166 country code](https://en.wikipedia.org/wiki/List_of_ISO_3166_country_codes).
              For example, Mexican Spanish is `es-MX`. See the
              [list of supported languages and language codes](https://dev.writer.com/api-reference/translation-api/language-support).

          text: The text to translate. Maximum of 100,000 words.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/translation",
            body=maybe_transform(
                {
                    "formality": formality,
                    "length_control": length_control,
                    "mask_profanity": mask_profanity,
                    "model": model,
                    "source_language_code": source_language_code,
                    "target_language_code": target_language_code,
                    "text": text,
                },
                translation_translate_params.TranslationTranslateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TranslationResponse,
        )


class AsyncTranslationResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTranslationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/writer/writer-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTranslationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTranslationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/writer/writer-python#with_streaming_response
        """
        return AsyncTranslationResourceWithStreamingResponse(self)

    async def translate(
        self,
        *,
        formality: bool,
        length_control: bool,
        mask_profanity: bool,
        model: Literal["palmyra-translate"],
        source_language_code: str,
        target_language_code: str,
        text: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TranslationResponse:
        """
        Translate text from one language to another.

        Args:
          formality: Whether to use formal or informal language in the translation. See the
              [list of languages that support formality](https://dev.writer.com/api-reference/translation-api/language-support#formality).
              If the language does not support formality, this parameter is ignored.

          length_control: Whether to control the length of the translated text. See the
              [list of languages that support length control](https://dev.writer.com/api-reference/translation-api/language-support#length-control).
              If the language does not support length control, this parameter is ignored.

          mask_profanity: Whether to mask profane words in the translated text. See the
              [list of languages that do not support profanity masking](https://dev.writer.com/api-reference/translation-api/language-support#profanity-masking).
              If the language does not support profanity masking, this parameter is ignored.

          model: The model to use for translation.

          source_language_code: The [ISO-639-1](https://en.wikipedia.org/wiki/List_of_ISO_639_language_codes)
              language code of the original text to translate. For example, `en` for English,
              `zh` for Chinese, `fr` for French, `es` for Spanish. If the language has a
              variant, the code appends the two-digit
              [ISO-3166 country code](https://en.wikipedia.org/wiki/List_of_ISO_3166_country_codes).
              For example, Mexican Spanish is `es-MX`. See the
              [list of supported languages and language codes](https://dev.writer.com/api-reference/translation-api/language-support).

          target_language_code: The [ISO-639-1](https://en.wikipedia.org/wiki/List_of_ISO_639_language_codes)
              language code of the target language for the translation. For example, `en` for
              English, `zh` for Chinese, `fr` for French, `es` for Spanish. If the language
              has a variant, the code appends the two-digit
              [ISO-3166 country code](https://en.wikipedia.org/wiki/List_of_ISO_3166_country_codes).
              For example, Mexican Spanish is `es-MX`. See the
              [list of supported languages and language codes](https://dev.writer.com/api-reference/translation-api/language-support).

          text: The text to translate. Maximum of 100,000 words.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/translation",
            body=await async_maybe_transform(
                {
                    "formality": formality,
                    "length_control": length_control,
                    "mask_profanity": mask_profanity,
                    "model": model,
                    "source_language_code": source_language_code,
                    "target_language_code": target_language_code,
                    "text": text,
                },
                translation_translate_params.TranslationTranslateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TranslationResponse,
        )


class TranslationResourceWithRawResponse:
    def __init__(self, translation: TranslationResource) -> None:
        self._translation = translation

        self.translate = to_raw_response_wrapper(
            translation.translate,
        )


class AsyncTranslationResourceWithRawResponse:
    def __init__(self, translation: AsyncTranslationResource) -> None:
        self._translation = translation

        self.translate = async_to_raw_response_wrapper(
            translation.translate,
        )


class TranslationResourceWithStreamingResponse:
    def __init__(self, translation: TranslationResource) -> None:
        self._translation = translation

        self.translate = to_streamed_response_wrapper(
            translation.translate,
        )


class AsyncTranslationResourceWithStreamingResponse:
    def __init__(self, translation: AsyncTranslationResource) -> None:
        self._translation = translation

        self.translate = async_to_streamed_response_wrapper(
            translation.translate,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel
from .shared.logprobs import Logprobs
from .chat_completion_message import ChatCompletionMessage

__all__ = ["ChatCompletionChoice"]


class ChatCompletionChoice(BaseModel):
    finish_reason: Literal["stop", "length", "content_filter", "tool_calls"]
    """Describes the condition under which the model ceased generating content.

    Common reasons include 'length' (reached the maximum output size), 'stop'
    (encountered a stop sequence), 'content_filter' (harmful content filtered out),
    or 'tool_calls' (encountered tool calls).
    """

    index: int
    """The index of the choice in the list of completions generated by the model."""

    message: ChatCompletionMessage
    """The chat completion message from the model.

    Note: this field is deprecated for streaming. Use `delta` instead.
    """

    logprobs: Optional[Logprobs] = None
    """Log probability information for the choice."""



from __future__ import annotations

import os
from typing import Any, Generic, Callable, Iterator, overload
from typing_extensions import Literal, TypeVar

# import rich
import httpx
import pytest
from respx import MockRouter
from pydantic import BaseModel
from inline_snapshot import external, snapshot, outsource

import writerai
from writerai import Writer, AsyncWriter
from writerai._utils import assert_signatures_in_sync

# from writerai._compat import model_copy
from writerai.lib.streaming.chat import (
    ContentDoneEvent,
    ChatCompletionStream,
    ChatCompletionStreamEvent,
    ChatCompletionStreamManager,
    # ParsedChatCompletionSnapshot,
)
from writerai.lib._parsing._completions import ResponseFormatT

from ._utils import print_obj
from ...conftest import base_url

_T = TypeVar("_T")

# all the snapshots in this file are auto-generated from the live API
#
# you can update them with
#
# `WRITER_LIVE=1 pytest --inline-snapshot=fix`


@pytest.mark.respx(base_url=base_url)
def test_parse_nothing(client: Writer, respx_mock: MockRouter, monkeypatch: pytest.MonkeyPatch) -> None:
    listener = _make_stream_snapshot_request(
        lambda c: c.chat.stream(
            model="palmyra-x-003-instruct",
            messages=[
                {
                    "role": "user",
                    "content": "What's the weather like in SF?",
                },
            ],
        ),
        content_snapshot=snapshot(external("e2aad469b71d*.writer")),
        openai_content_snapshot=snapshot(external("e2aad469b71d*.openai")),
        mock_client=client,
        respx_mock=respx_mock,
    )

    assert print_obj(listener.stream.get_final_completion().choices, monkeypatch) == snapshot(
        """\
[
    ParsedChatCompletionChoice[NoneType](
        finish_reason='stop',
        index=0,
        logprobs=None,
        message=ParsedChatCompletionMessage[NoneType](
            content="I'm unable to provide real-time weather updates. To get the current weather in San Francisco, I 
recommend checking a reliable weather website or a weather app.",
            graph_data=None,
            llm_data=None,
            parsed=None,
            refusal=None,
            role='assistant',
            tool_calls=None,
            translation_data=None,
            web_search_data=None
        )
    )
]
"""
    )
    assert print_obj(listener.get_event_by_type("content.done"), monkeypatch) == snapshot(
        """\
ContentDoneEvent[NoneType](
    content="I'm unable to provide real-time weather updates. To get the current weather in San Francisco, I recommend 
checking a reliable weather website or a weather app.",
    parsed=None,
    type='content.done'
)
"""
    )


# @pytest.mark.respx(base_url=base_url)
# def test_parse_pydantic_model(client: Writer, respx_mock: MockRouter, monkeypatch: pytest.MonkeyPatch) -> None:
#     class Location(BaseModel):
#         city: str
#         temperature: float
#         units: Literal["c", "f"]

#     done_snapshots: list[ParsedChatCompletionSnapshot] = []

#     def on_event(stream: ChatCompletionStream[Location], event: ChatCompletionStreamEvent[Location]) -> None:
#         if event.type == "content.done":
#             done_snapshots.append(model_copy(stream.current_completion_snapshot, deep=True))

#     listener = _make_stream_snapshot_request(
#         lambda c: c.chat.stream(
#             model="palmyra-x-003-instruct",
#             messages=[
#                 {
#                     "role": "user",
#                     "content": "What's the weather like in SF?",
#                 },
#             ],
#             # response_format=Location,
#         ),
#         content_snapshot=snapshot(external("7e5ea4d12e7c*_bin")),
#         mock_client=client,
#         respx_mock=respx_mock,
#         on_event=on_event,
#     )

#     assert len(done_snapshots) == 1
#     assert isinstance(done_snapshots[0].choices[0].message.parsed, Location)

#     for event in reversed(listener.events):
#         if event.type == "content.delta":
#             data = cast(Any, event.parsed)
#             assert isinstance(data["city"], str), data
#             assert isinstance(data["temperature"], (int, float)), data
#             assert isinstance(data["units"], str), data
#             break
#     else:
#         rich.print(listener.events)
#         raise AssertionError("Did not find a `content.delta` event")

#     assert print_obj(listener.stream.get_final_completion(), monkeypatch) == snapshot(
#         """\
# ParsedChatCompletion[Location](
#     choices=[
#         ParsedChatCompletionChoice[Location](
#             finish_reason='stop',
#             index=0,
#             logprobs=None,
#             message=ParsedChatCompletionMessage[Location](
#                 content='{"city":"San Francisco","temperature":61,"units":"f"}',
#                 graph_data=None,
#                 parsed=Location(city='San Francisco', temperature=61.0, units='f'),
#                 refusal=None,
#                 role='assistant',
#                 tool_calls=[],
#                 translation_data=None
#             )
#         )
#     ],
#     created=1727346169,
#     id='chatcmpl-ABfw1e5abtU8OwGr15vOreYVb2MiF',
#     model='palmyra-x-003-instruct',
#     object='chat.completion',
#     service_tier=None,
#     system_fingerprint='fp_5050236cbd',
#     usage=CompletionUsage(
#         completion_tokens=14,
#         completion_tokens_details=CompletionTokensDetails(reasoning_tokens=0),
#         prompt_tokens=79,
#         total_tokens=93
#     )
# )
# """
#     )
#     assert print_obj(listener.get_event_by_type("content.done"), monkeypatch) == snapshot(
#         """\
# ContentDoneEvent[Location](
#     content='{"city":"San Francisco","temperature":61,"units":"f"}',
#     parsed=Location(city='San Francisco', temperature=61.0, units='f'),
#     type='content.done'
# )
# """
#     )


# @pytest.mark.respx(base_url=base_url)
# def test_parse_pydantic_model_multiple_choices(
#     client: Writer, respx_mock: MockRouter, monkeypatch: pytest.MonkeyPatch
# ) -> None:
#     class Location(BaseModel):
#         city: str
#         temperature: float
#         units: Literal["c", "f"]

#     listener = _make_stream_snapshot_request(
#         lambda c: c.chat.stream(
#             model="palmyra-x-003-instruct",
#             messages=[
#                 {
#                     "role": "user",
#                     "content": "What's the weather like in SF?",
#                 },
#             ],
#             n=3,
#             response_format=Location,
#         ),
#         content_snapshot=snapshot(external("a491adda08c3*_bin")),
#         mock_client=client,
#         respx_mock=respx_mock,
#     )

#     assert [e.type for e in listener.events] == snapshot(
#         [
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.delta",
#             "chunk",
#             "content.done",
#             "chunk",
#             "content.done",
#             "chunk",
#             "content.done",
#             "chunk",
#         ]
#     )
#     assert print_obj(listener.stream.get_final_completion().choices, monkeypatch) == snapshot(
#         """\
# [
#     ParsedChatCompletionChoice[Location](
#         finish_reason='stop',
#         index=0,
#         logprobs=None,
#         message=ParsedChatCompletionMessage[Location](
#             content='{"city":"San Francisco","temperature":65,"units":"f"}',
#             graph_data=None,
#             parsed=Location(city='San Francisco', temperature=65.0, units='f'),
#             refusal=None,
#             role='assistant',
#             tool_calls=[],
#             translation_data=None
#         )
#     ),
#     ParsedChatCompletionChoice[Location](
#         finish_reason='stop',
#         index=1,
#         logprobs=None,
#         message=ParsedChatCompletionMessage[Location](
#             content='{"city":"San Francisco","temperature":61,"units":"f"}',
#             graph_data=None,
#             parsed=Location(city='San Francisco', temperature=61.0, units='f'),
#             refusal=None,
#             role='assistant',
#             tool_calls=[],
#             translation_data=None
#         )
#     ),
#     ParsedChatCompletionChoice[Location](
#         finish_reason='stop',
#         index=2,
#         logprobs=None,
#         message=ParsedChatCompletionMessage[Location](
#             content='{"city":"San Francisco","temperature":59,"units":"f"}',
#             graph_data=None,
#             parsed=Location(city='San Francisco', temperature=59.0, units='f'),
#             refusal=None,
#             role='assistant',
#             tool_calls=[],
#             translation_data=None
#         )
#     )
# ]
# """
#     )


# @pytest.mark.respx(base_url=base_url)
# def test_parse_max_tokens_reached(client: Writer, respx_mock: MockRouter) -> None:
#     class Location(BaseModel):
#         city: str
#         temperature: float
#         units: Literal["c", "f"]
#
#     with pytest.raises(writerai.LengthFinishReasonError):
#         _make_stream_snapshot_request(
#             lambda c: c.chat.stream(
#                 model="palmyra-x-003-instruct",
#                 messages=[
#                     {
#                         "role": "user",
#                         "content": "What's the weather like in SF?",
#                     },
#                 ],
#                 max_tokens=1,
#                 response_format=Location,
#             ),
#             content_snapshot=snapshot(external("4cc50a6135d2*_bin")),
#             mock_client=client,
#             respx_mock=respx_mock,
#         )


# @pytest.mark.respx(base_url=base_url)
# def test_parse_pydantic_model_refusal(client: Writer, respx_mock: MockRouter, monkeypatch: pytest.MonkeyPatch) -> None:
#     class Location(BaseModel):
#         city: str
#         temperature: float
#         units: Literal["c", "f"]
#
#     listener = _make_stream_snapshot_request(
#         lambda c: c.chat.stream(
#             model="palmyra-x-003-instruct",
#             messages=[
#                 {
#                     "role": "user",
#                     "content": "How do I make anthrax?",
#                 },
#             ],
#             response_format=Location,
#         ),
#         content_snapshot=snapshot(external("173417d55340*_bin")),
#         mock_client=client,
#         respx_mock=respx_mock,
#     )
#
#     assert print_obj(listener.get_event_by_type("refusal.done"), monkeypatch) == snapshot("""\
# RefusalDoneEvent(refusal="I'm sorry, I can't assist with that request.", type='refusal.done')
# """)
#
#     assert print_obj(listener.stream.get_final_completion().choices, monkeypatch) == snapshot(
#         """\
# [
#     ParsedChatCompletionChoice[Location](
#         finish_reason='stop',
#         index=0,
#         logprobs=None,
#         message=ParsedChatCompletionMessage[Location](
#             content=None,
#             graph_data=None,
#             parsed=None,
#             refusal="I'm sorry, I can't assist with that request.",
#             role='assistant',
#             tool_calls=[],
#             translation_data=None
#         )
#     )
# ]
# """
#     )


@pytest.mark.respx(base_url=base_url)
def test_content_logprobs_events(client: Writer, respx_mock: MockRouter, monkeypatch: pytest.MonkeyPatch) -> None:
    listener = _make_stream_snapshot_request(
        lambda c: c.chat.stream(
            model="palmyra-x-003-instruct",
            messages=[
                {
                    "role": "user",
                    "content": "Say foo",
                },
            ],
            logprobs=True,
        ),
        content_snapshot=snapshot(external("83b060bae42e*.writer")),
        openai_content_snapshot=snapshot(external("83b060bae42e*.openai")),
        mock_client=client,
        respx_mock=respx_mock,
    )

    assert print_obj([e for e in listener.events if e.type.startswith("logprobs")], monkeypatch) == snapshot("""\
[
    LogprobsContentDeltaEvent(
        content=[LogprobsToken(bytes=[70, 111, 111], logprob=-0.0025094282, token='Foo', top_logprobs=[])],
        snapshot=[LogprobsToken(bytes=[70, 111, 111], logprob=-0.0025094282, token='Foo', top_logprobs=[])],
        type='logprobs.content.delta'
    ),
    LogprobsContentDeltaEvent(
        content=[LogprobsToken(bytes=[33], logprob=-0.26638845, token='!', top_logprobs=[])],
        snapshot=[
            LogprobsToken(bytes=[70, 111, 111], logprob=-0.0025094282, token='Foo', top_logprobs=[]),
            LogprobsToken(bytes=[33], logprob=-0.26638845, token='!', top_logprobs=[])
        ],
        type='logprobs.content.delta'
    ),
    LogprobsContentDoneEvent(
        content=[
            LogprobsToken(bytes=[70, 111, 111], logprob=-0.0025094282, token='Foo', top_logprobs=[]),
            LogprobsToken(bytes=[33], logprob=-0.26638845, token='!', top_logprobs=[])
        ],
        type='logprobs.content.done'
    )
]
""")

    assert print_obj(listener.stream.get_final_completion().choices, monkeypatch) == snapshot("""\
[
    ParsedChatCompletionChoice[NoneType](
        finish_reason='stop',
        index=0,
        logprobs=Logprobs(
            content=[
                LogprobsToken(bytes=[70, 111, 111], logprob=-0.0025094282, token='Foo', top_logprobs=[]),
                LogprobsToken(bytes=[33], logprob=-0.26638845, token='!', top_logprobs=[])
            ],
            refusal=None
        ),
        message=ParsedChatCompletionMessage[NoneType](
            content='Foo!',
            graph_data=None,
            llm_data=None,
            parsed=None,
            refusal=None,
            role='assistant',
            tool_calls=None,
            translation_data=None,
            web_search_data=None
        )
    )
]
""")


@pytest.mark.respx(base_url=base_url)
def test_refusal_logprobs_events(client: Writer, respx_mock: MockRouter, monkeypatch: pytest.MonkeyPatch) -> None:
    #     class Location(BaseModel):
    #         city: str
    #         temperature: float
    #         units: Literal["c", "f"]

    listener = _make_stream_snapshot_request(
        lambda c: c.chat.stream(
            model="palmyra-x-003-instruct",
            messages=[
                {
                    "role": "user",
                    "content": "How do I make anthrax?",
                },
            ],
            logprobs=True,
            # response_format=Location,
        ),
        content_snapshot=snapshot(external("569c877e6942*.writer")),
        openai_content_snapshot=snapshot(external("569c877e6942*.openai")),
        mock_client=client,
        respx_mock=respx_mock,
    )

    assert print_obj([e.type for e in listener.events if e.type.startswith("logprobs")], monkeypatch) == snapshot("""\
[
    'logprobs.refusal.delta',
    'logprobs.refusal.delta',
    'logprobs.refusal.delta',
    'logprobs.refusal.delta',
    'logprobs.refusal.delta',
    'logprobs.refusal.delta',
    'logprobs.refusal.delta',
    'logprobs.refusal.delta',
    'logprobs.refusal.delta',
    'logprobs.refusal.delta',
    'logprobs.refusal.delta',
    'logprobs.refusal.done'
]
""")

    assert print_obj(listener.stream.get_final_completion().choices, monkeypatch) == snapshot("""\
[
    ParsedChatCompletionChoice[NoneType](
        finish_reason='stop',
        index=0,
        logprobs=Logprobs(
            content=None,
            refusal=[
                LogprobsToken(bytes=[73, 39, 109], logprob=-0.0012038043, token="I'm", top_logprobs=[]),
                LogprobsToken(bytes=[32, 118, 101, 114, 121], logprob=-0.8438816, token=' very', top_logprobs=[]),
                LogprobsToken(
                    bytes=[32, 115, 111, 114, 114, 121],
                    logprob=-3.4121115e-06,
                    token=' sorry',
                    top_logprobs=[]
                ),
                LogprobsToken(bytes=[44], logprob=-3.3809047e-05, token=',', top_logprobs=[]),
                LogprobsToken(bytes=[32, 98, 117, 116], logprob=-0.038048144, token=' but', top_logprobs=[]),
                LogprobsToken(bytes=[32, 73], logprob=-0.0016109125, token=' I', top_logprobs=[]),
                LogprobsToken(
                    bytes=[32, 99, 97, 110, 39, 116],
                    logprob=-0.0073532974,
                    token=" can't",
                    top_logprobs=[]
                ),
                LogprobsToken(
                    bytes=[32, 97, 115, 115, 105, 115, 116],
                    logprob=-0.0020837625,
                    token=' assist',
                    top_logprobs=[]
                ),
                LogprobsToken(bytes=[32, 119, 105, 116, 104], logprob=-0.00318354, token=' with', top_logprobs=[]),
                LogprobsToken(bytes=[32, 116, 104, 97, 116], logprob=-0.0017186158, token=' that', top_logprobs=[]),
                LogprobsToken(bytes=[46], logprob=-0.57687104, token='.', top_logprobs=[])
            ]
        ),
        message=ParsedChatCompletionMessage[NoneType](
            content=None,
            graph_data=None,
            llm_data=None,
            parsed=None,
            refusal="I'm very sorry, but I can't assist with that.",
            role='assistant',
            tool_calls=None,
            translation_data=None,
            web_search_data=None
        )
    )
]
""")


@pytest.mark.respx(base_url=base_url)
def test_parse_pydantic_tool(client: Writer, respx_mock: MockRouter, monkeypatch: pytest.MonkeyPatch) -> None:
    class GetWeatherArgs(BaseModel):
        city: str
        country: str
        units: Literal["c", "f"] = "c"

    listener = _make_stream_snapshot_request(
        lambda c: c.chat.stream(
            model="palmyra-x-003-instruct",
            messages=[
                {
                    "role": "user",
                    "content": "What's the weather like in Edinburgh?",
                },
            ],
            tools=[
                writerai.pydantic_function_tool(GetWeatherArgs),
            ],
        ),
        content_snapshot=snapshot(external("c6aa7e397b71*.writer")),
        openai_content_snapshot=snapshot(external("c6aa7e397b71*.openai")),
        mock_client=client,
        respx_mock=respx_mock,
    )

    assert print_obj(listener.stream.current_completion_snapshot.choices, monkeypatch) == snapshot(
        """\
[
    ParsedChatCompletionChoice[object](
        finish_reason='tool_calls',
        index=0,
        logprobs=None,
        message=ParsedChatCompletionMessage[object](
            content=None,
            graph_data=None,
            llm_data=None,
            parsed=None,
            refusal=None,
            role='assistant',
            tool_calls=[
                ParsedFunctionToolCall(
                    function=ParsedFunction(
                        arguments='{"city":"Edinburgh","country":"UK","units":"c"}',
                        name='GetWeatherArgs',
                        parsed_arguments=GetWeatherArgs(city='Edinburgh', country='UK', units='c')
                    ),
                    id='call_c91SqDXlYFuETYv8mUHzz6pp',
                    index=0,
                    type='function'
                )
            ],
            translation_data=None,
            web_search_data=None
        )
    )
]
"""
    )

    assert print_obj(listener.stream.get_final_completion().choices, monkeypatch) == snapshot(
        """\
[
    ParsedChatCompletionChoice[NoneType](
        finish_reason='tool_calls',
        index=0,
        logprobs=None,
        message=ParsedChatCompletionMessage[NoneType](
            content=None,
            graph_data=None,
            llm_data=None,
            parsed=None,
            refusal=None,
            role='assistant',
            tool_calls=[
                ParsedFunctionToolCall(
                    function=ParsedFunction(
                        arguments='{"city":"Edinburgh","country":"UK","units":"c"}',
                        name='GetWeatherArgs',
                        parsed_arguments=GetWeatherArgs(city='Edinburgh', country='UK', units='c')
                    ),
                    id='call_c91SqDXlYFuETYv8mUHzz6pp',
                    index=0,
                    type='function'
                )
            ],
            translation_data=None,
            web_search_data=None
        )
    )
]
"""
    )


@pytest.mark.respx(base_url=base_url)
def test_parse_multiple_pydantic_tools(client: Writer, respx_mock: MockRouter, monkeypatch: pytest.MonkeyPatch) -> None:
    class GetWeatherArgs(BaseModel):
        """Get the temperature for the given country/city combo"""

        city: str
        country: str
        units: Literal["c", "f"] = "c"

    class GetStockPrice(BaseModel):
        ticker: str
        exchange: str

    listener = _make_stream_snapshot_request(
        lambda c: c.chat.stream(
            model="palmyra-x-003-instruct",
            messages=[
                {
                    "role": "user",
                    "content": "What's the weather like in Edinburgh?",
                },
                {
                    "role": "user",
                    "content": "What's the price of AAPL?",
                },
            ],
            tools=[
                writerai.pydantic_function_tool(GetWeatherArgs),
                writerai.pydantic_function_tool(
                    GetStockPrice, name="get_stock_price", description="Fetch the latest price for a given ticker"
                ),
            ],
        ),
        content_snapshot=snapshot(external("f82268f2fefd*.writer")),
        openai_content_snapshot=snapshot(external("f82268f2fefd*.openai")),
        mock_client=client,
        respx_mock=respx_mock,
    )

    assert print_obj(listener.stream.current_completion_snapshot.choices, monkeypatch) == snapshot(
        """\
[
    ParsedChatCompletionChoice[object](
        finish_reason='tool_calls',
        index=0,
        logprobs=None,
        message=ParsedChatCompletionMessage[object](
            content=None,
            graph_data=None,
            llm_data=None,
            parsed=None,
            refusal=None,
            role='assistant',
            tool_calls=[
                ParsedFunctionToolCall(
                    function=ParsedFunction(
                        arguments='{"city": "Edinburgh", "country": "GB", "units": "c"}',
                        name='GetWeatherArgs',
                        parsed_arguments=GetWeatherArgs(city='Edinburgh', country='GB', units='c')
                    ),
                    id='call_JMW1whyEaYG438VE1OIflxA2',
                    index=0,
                    type='function'
                ),
                ParsedFunctionToolCall(
                    function=ParsedFunction(
                        arguments='{"ticker": "AAPL", "exchange": "NASDAQ"}',
                        name='get_stock_price',
                        parsed_arguments=GetStockPrice(exchange='NASDAQ', ticker='AAPL')
                    ),
                    id='call_DNYTawLBoN8fj3KN6qU9N1Ou',
                    index=1,
                    type='function'
                )
            ],
            translation_data=None,
            web_search_data=None
        )
    )
]
"""
    )
    completion = listener.stream.get_final_completion()
    assert print_obj(completion.choices[0].message.tool_calls, monkeypatch) == snapshot(
        """\
[
    ParsedFunctionToolCall(
        function=ParsedFunction(
            arguments='{"city": "Edinburgh", "country": "GB", "units": "c"}',
            name='GetWeatherArgs',
            parsed_arguments=GetWeatherArgs(city='Edinburgh', country='GB', units='c')
        ),
        id='call_JMW1whyEaYG438VE1OIflxA2',
        index=0,
        type='function'
    ),
    ParsedFunctionToolCall(
        function=ParsedFunction(
            arguments='{"ticker": "AAPL", "exchange": "NASDAQ"}',
            name='get_stock_price',
            parsed_arguments=GetStockPrice(exchange='NASDAQ', ticker='AAPL')
        ),
        id='call_DNYTawLBoN8fj3KN6qU9N1Ou',
        index=1,
        type='function'
    )
]
"""
    )


# @pytest.mark.respx(base_url=base_url)
# def test_parse_strict_tools(client: Writer, respx_mock: MockRouter, monkeypatch: pytest.MonkeyPatch) -> None:
#     listener = _make_stream_snapshot_request(
#         lambda c: c.chat.stream(
#             model="palmyra-x-003-instruct",
#             messages=[
#                 {
#                     "role": "user",
#                     "content": "What's the weather like in SF?",
#                 },
#             ],
#             tools=[
#                 {
#                     "type": "function",
#                     "function": {
#                         "name": "get_weather",
#                         "parameters": {
#                             "type": "object",
#                             "properties": {
#                                 "city": {"type": "string"},
#                                 "state": {"type": "string"},
#                             },
#                             "required": [
#                                 "city",
#                                 "state",
#                             ],
#                             "additionalProperties": False,
#                         },
#                         "strict": True,
#                     },
#                 }
#             ],
#         ),
#         content_snapshot=snapshot(external("a247c49c5fcd*_bin")),
#         mock_client=client,
#         respx_mock=respx_mock,
#     )

#     assert print_obj(listener.stream.current_completion_snapshot.choices, monkeypatch) == snapshot(
#         """\
# [
#     ParsedChatCompletionChoice[object](
#         finish_reason='tool_calls',
#         index=0,
#         logprobs=None,
#         message=ParsedChatCompletionMessage[object](
#             content=None,
#             graph_data=None,
#             parsed=None,
#             refusal=None,
#             role='assistant',
#             tool_calls=[
#                 ParsedFunctionToolCall(
#                     function=ParsedFunction(
#                         arguments='{"city":"San Francisco","state":"CA"}',
#                         name='get_weather',
#                         parsed_arguments={'city': 'San Francisco', 'state': 'CA'}
#                     ),
#                     id='call_CTf1nWJLqSeRgDqaCG27xZ74',
#                     index=0,
#                     type='function'
#                 )
#             ]
#         )
#     )
# ]
# """
#     )


@pytest.mark.respx(base_url=base_url)
def test_non_pydantic_response_format(client: Writer, respx_mock: MockRouter, monkeypatch: pytest.MonkeyPatch) -> None:
    listener = _make_stream_snapshot_request(
        lambda c: c.chat.stream(
            model="palmyra-x-003-instruct",
            messages=[
                {
                    "role": "user",
                    "content": "What's the weather like in SF? Give me any JSON back",
                },
            ],
            # response_format={"type": "json_object"},
        ),
        content_snapshot=snapshot(external("d61558011839*.writer")),
        openai_content_snapshot=snapshot(external("d61558011839*.openai")),
        mock_client=client,
        respx_mock=respx_mock,
    )

    assert print_obj(listener.stream.get_final_completion().choices, monkeypatch) == snapshot(
        """\
[
    ParsedChatCompletionChoice[NoneType](
        finish_reason='stop',
        index=0,
        logprobs=None,
        message=ParsedChatCompletionMessage[NoneType](
            content='\\n  {\\n    "location": "San Francisco, CA",\\n    "weather": {\\n      "temperature": "18°C",\\n      
"condition": "Partly Cloudy",\\n      "humidity": "72%",\\n      "windSpeed": "15 km/h",\\n      "windDirection": "NW"\\n   
},\\n    "forecast": [\\n      {\\n        "day": "Monday",\\n        "high": "20°C",\\n        "low": "14°C",\\n        
"condition": "Sunny"\\n      },\\n      {\\n        "day": "Tuesday",\\n        "high": "19°C",\\n        "low": "15°C",\\n   
"condition": "Mostly Cloudy"\\n      },\\n      {\\n        "day": "Wednesday",\\n        "high": "18°C",\\n        "low": 
"14°C",\\n        "condition": "Cloudy"\\n      }\\n    ]\\n  }\\n',
            graph_data=None,
            llm_data=None,
            parsed=None,
            refusal=None,
            role='assistant',
            tool_calls=None,
            translation_data=None,
            web_search_data=None
        )
    )
]
"""
    )


@pytest.mark.respx(base_url=base_url)
def test_allows_non_strict_tools_but_no_parsing(
    client: Writer, respx_mock: MockRouter, monkeypatch: pytest.MonkeyPatch
) -> None:
    listener = _make_stream_snapshot_request(
        lambda c: c.chat.stream(
            model="palmyra-x-003-instruct",
            messages=[{"role": "user", "content": "what's the weather in NYC?"}],
            tools=[
                {
                    "type": "function",
                    "function": {
                        "name": "get_weather",
                        "parameters": {"type": "object", "properties": {"city": {"type": "string"}}},
                    },
                }
            ],
        ),
        content_snapshot=snapshot(external("2018feb66ae1*.writer")),
        openai_content_snapshot=snapshot(external("2018feb66ae1*.openai")),
        mock_client=client,
        respx_mock=respx_mock,
    )

    assert print_obj(listener.get_event_by_type("tool_calls.function.arguments.done"), monkeypatch) == snapshot("""\
FunctionToolCallArgumentsDoneEvent(
    arguments='{"city":"New York City"}',
    index=0,
    name='get_weather',
    parsed_arguments=None,
    tool_call_snapshot_id='call_4XzlGBLtUe9dy3GVNV4jhq7h',
    type='tool_calls.function.arguments.done'
)
""")

    assert print_obj(listener.stream.get_final_completion().choices, monkeypatch) == snapshot(
        """\
[
    ParsedChatCompletionChoice[NoneType](
        finish_reason='tool_calls',
        index=0,
        logprobs=None,
        message=ParsedChatCompletionMessage[NoneType](
            content=None,
            graph_data=None,
            llm_data=None,
            parsed=None,
            refusal=None,
            role='assistant',
            tool_calls=[
                ParsedFunctionToolCall(
                    function=ParsedFunction(
                        arguments='{"city":"New York City"}',
                        name='get_weather',
                        parsed_arguments=None
                    ),
                    id='call_4XzlGBLtUe9dy3GVNV4jhq7h',
                    index=0,
                    type='function'
                )
            ],
            translation_data=None,
            web_search_data=None
        )
    )
]
"""
    )


@pytest.mark.parametrize("sync", [True, False], ids=["sync", "async"])
def test_stream_method_in_sync(sync: bool, client: Writer, async_client: AsyncWriter) -> None:
    checking_client: Writer | AsyncWriter = client if sync else async_client

    assert_signatures_in_sync(
        checking_client.chat.chat,
        checking_client.chat.stream,
        exclude_params={"response_format", "stream"},
    )


class StreamListener(Generic[ResponseFormatT]):
    def __init__(self, stream: ChatCompletionStream[ResponseFormatT]) -> None:
        self.stream = stream
        self.events: list[ChatCompletionStreamEvent[ResponseFormatT]] = []

    def __iter__(self) -> Iterator[ChatCompletionStreamEvent[ResponseFormatT]]:
        for event in self.stream:
            self.events.append(event)
            yield event

    @overload
    def get_event_by_type(self, event_type: Literal["content.done"]) -> ContentDoneEvent[ResponseFormatT] | None: ...

    @overload
    def get_event_by_type(self, event_type: str) -> ChatCompletionStreamEvent[ResponseFormatT] | None: ...

    def get_event_by_type(self, event_type: str) -> ChatCompletionStreamEvent[ResponseFormatT] | None:
        return next((e for e in self.events if e.type == event_type), None)


def _make_stream_snapshot_request(
    func: Callable[[Writer], ChatCompletionStreamManager[ResponseFormatT]],
    *,
    content_snapshot: Any,
    openai_content_snapshot: Any,
    respx_mock: MockRouter,
    mock_client: Writer,
    on_event: Callable[[ChatCompletionStream[ResponseFormatT], ChatCompletionStreamEvent[ResponseFormatT]], Any]
    | None = None,
) -> StreamListener[ResponseFormatT]:
    def mock_response(snapshot: Any) -> Writer:
        respx_mock.post("/v1/chat").mock(
            return_value=httpx.Response(
                200,
                content=snapshot._old_value._load_value(),
                headers={"content-type": "text/event-stream"},
            )
        )
        return mock_client

    def consume_stream(client: Writer) -> StreamListener[ResponseFormatT]:
        with func(client) as stream:
            listener = StreamListener(stream)

            for event in listener:
                if on_event:
                    on_event(stream, event)
            return listener

    # Test against Writer's content snapshot
    live = os.environ.get("WRITER_LIVE") == "1"
    if live:

        def _on_response(response: httpx.Response) -> None:
            # update Writer's content snapshot
            assert outsource(response.read(), suffix=".writer") == content_snapshot

        respx_mock.stop()

        client = Writer(
            http_client=httpx.Client(
                event_hooks={
                    "response": [_on_response],
                }
            )
        )
    else:
        client = mock_response(content_snapshot)

    writer_listener = consume_stream(client)

    if live:
        client.close()

    # Ensure compatibility with OpenAI's content snapshot
    client = mock_response(openai_content_snapshot)
    consume_stream(client)

    return writer_listener



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .comprehend_medical_params import ComprehendMedicalParams as ComprehendMedicalParams
from .comprehend_medical_response import ComprehendMedicalResponse as ComprehendMedicalResponse
